pub enum AutorefOrPtrAdjustment {
Autoref {
mutbl: Mutability,
unsize: bool,
},
ToConstPtr,
}Expand description
When adjusting a receiver we often want to do one of
- Add a
&(or&mut), converting the receiver fromTto&T(or&mut T) - If the receiver has type
*mut T, convert it to*const T
This type tells us which one to do.
Note that in principle we could do both at the same time. For example, when the receiver has
type T, we could autoref it to &T, then convert to *const T. Or, when it has type *mut T, we could convert it to *const T, then autoref to &*const T. However, currently we do
(at most) one of these. Either the receiver has type T and we convert it to &T (or with
mut), or it has type *mut T and we convert it to *const T.
Variants
Autoref
Receiver has type T, add & or &mut (if T is mut), and maybe also “unsize” it.
Unsizing is used to convert a [T; N] to [T], which only makes sense when autorefing.
Fields

mutbl: MutabilityToConstPtr
Receiver has type *mut T, convert to *const T
Implementations
Sourceimpl AutorefOrPtrAdjustment
impl AutorefOrPtrAdjustment
fn get_unsize(&self) -> bool