fn is_relevant_kind_for_mode(
mode: Mode,
kind: AssocItemId,
) -> Option<CandidateId>Expand description
Determine if the given associated item type is relevant in the current context.
fn is_relevant_kind_for_mode(
mode: Mode,
kind: AssocItemId,
) -> Option<CandidateId>Determine if the given associated item type is relevant in the current context.