struct MethodAutoderefStepsResult<'db> {
pub steps: SmallVec<[CandidateStep<'db>; 3]>,
pub opt_bad_ty: Option<MethodAutoderefBadTy<'db>>,
pub reached_recursion_limit: bool,
}Fields
steps: SmallVec<[CandidateStep<'db>; 3]>The valid autoderef steps that could be found by following a chain
of Receiver<Target=T> or Deref<Target=T> trait implementations.
opt_bad_ty: Option<MethodAutoderefBadTy<'db>>If Some(T), a type autoderef reported an error on.
reached_recursion_limit: boolIf true, steps has been truncated due to reaching the
recursion limit.