struct PickConstraintsForShadowed {
autoderefs: usize,
receiver_steps: Option<usize>,
def_id: CandidateId,
}Expand description
Criteria to apply when searching for a given Pick. This is used during the search for potentially shadowed methods to ensure we don’t search more candidates than strictly necessary.
Fields
autoderefs: usizereceiver_steps: Option<usize>def_id: CandidateIdImplementations
Sourceimpl PickConstraintsForShadowed
impl PickConstraintsForShadowed
fn may_shadow_based_on_autoderefs(&self, autoderefs: usize) -> bool
fn candidate_may_shadow(&self, candidate: &Candidate<'_>) -> bool